NEBTreeMap

Struct NEBTreeMap 

pub struct NEBTreeMap<K, V> { /* private fields */ }
A non-empty, growable BTreeMap.

use nonempty_collections::nebtm;

let m = nebtm!["elves" => 3000, "orcs" => 10000];
assert_eq!(2, m.len().get());

Implementations

Source

impl<K, V> NEBTreeMap<K, V>
where K: Ord,

pub fn new(k: K, v: V) -> NEBTreeMap<K, V>

Creates a new NEBTreeMap with a single element.

Source

impl<K, V> NEBTreeMap<K, V>

pub fn try_from_map(map: BTreeMap<K, V>) -> Option<Self>

Attempt a conversion from BTreeMap, consuming the given BTreeMap. Will return None if the BTreeMap is empty.

use std::collections::*;

use nonempty_collections::*;

let mut map = BTreeMap::new();
map.extend([("a", 1), ("b", 2)]);
assert_eq!(Some(nebtm! {"a" => 1, "b" => 2}), NEBTreeMap::try_from_map(map));
let map: BTreeMap<(), ()> = BTreeMap::new();
assert_eq!(None, NEBTreeMap::try_from_map(map));
pub fn iter(&self) -> Iter<'_, K, V>

Returns a regular iterator over the entries in this non-empty map.

For a NonEmptyIterator see Self::nonempty_iter().

pub fn iter_mut(&mut self) -> IterMut<'_, K, V>

Returns a regular mutable iterator over the entries in this non-empty map.

For a NonEmptyIterator see Self::nonempty_iter_mut().

pub fn nonempty_iter(&self) -> Iter<'_, K, V>

An iterator visiting all elements in arbitrary order. The iterator element type is (&'a K, &'a V).

pub fn nonempty_iter_mut(&mut self) -> IterMut<'_, K, V>

An iterator visiting all elements in arbitrary order. The iterator element type is (&'a K, &'a mut V).

Panics

If you manually advance this iterator until empty and then call first, you’re in for a surprise.

pub fn keys(&self) -> Keys<'_, K, V>

An iterator visiting all keys in arbitrary order. The iterator element type is &'a K.

use nonempty_collections::*;

let m = nebtm!["Valmar" => "Vanyar", "Tirion" => "Noldor", "Alqualondë" => "Teleri"];
let mut v: NEVec<_> = m.keys().collect();
v.sort();
assert_eq!(nev![&"Alqualondë", &"Tirion", &"Valmar"], v);
pub fn len(&self) -> NonZeroUsize

Returns the number of elements in the map. Always 1 or more.

use nonempty_collections::nebtm;

let m = nebtm!["a" => 1, "b" => 2];
assert_eq!(2, m.len().get());
pub fn values(&self) -> Values<'_, K, V>

An iterator visiting all values in arbitrary order. The iterator element type is &'a V.

use nonempty_collections::*;

let m = nebtm!["Valmar" => "Vanyar", "Tirion" => "Noldor", "Alqualondë" => "Teleri"];
let mut v: NEVec<_> = m.values().collect();
v.sort();
assert_eq!(nev![&"Noldor", &"Teleri", &"Vanyar"], v);
Source

impl<K, V> NEBTreeMap<K, V>
where K: Ord,

pub fn contains_key<Q>(&self, k: &Q) -> bool
where K: Borrow<Q>, Q: Ord + ?Sized,

Returns true if the map contains a value.

use nonempty_collections::nebtm;

let m = nebtm!["Jack" => 8];
assert!(m.contains_key("Jack"));
assert!(!m.contains_key("Colin"));
pub fn get<Q>(&self, k: &Q) -> Option<&V>
where K: Borrow<Q>, Q: Ord + ?Sized,

Returns a reference to the value corresponding to the key.

The key may be any borrowed form of the map’s value type, but Hash and Eq on the borrowed form must match those for the key type.

use nonempty_collections::nebtm;

let m = nebtm!["silmarils" => 3];
assert_eq!(Some(&3), m.get("silmarils"));
assert_eq!(None, m.get("arkenstone"));
pub fn get_key_value<Q>(&self, k: &Q) -> Option<(&K, &V)>
where K: Borrow<Q>, Q: Ord + ?Sized,

Returns the key-value pair corresponding to the key.

The key may be any borrowed form of the map’s value type, but Hash and Eq on the borrowed form must match those for the key type.

use nonempty_collections::nebtm;

let m = nebtm!["silmarils" => 3];
assert_eq!(Some((&"silmarils", &3)), m.get_key_value("silmarils"));
assert_eq!(None, m.get_key_value("arkenstone"));
pub fn get_mut<Q>(&mut self, k: &Q) -> Option<&mut V>
where K: Borrow<Q>, Q: Ord + ?Sized,

Returns a reference to the value corresponding to the key.

The key may be any borrowed form of the map’s value type, but Hash and Eq on the borrowed form must match those for the key type.

use nonempty_collections::nebtm;

let mut m = nebtm!["silmarils" => 3];
let mut v = m.get_mut("silmarils").unwrap();

// And thus it came to pass that the Silmarils found their long homes:
// one in the airs of heaven, and one in the fires of the heart of the
// world, and one in the deep waters.
*v -= 3;

assert_eq!(Some(&0), m.get("silmarils"));
pub fn insert(&mut self, k: K, v: V) -> Option<V>

Insert a key-value pair into the map.

If the map did not have this present, None is returned.

If the map did have this key present, the value is updated, and the old value is returned. The key is not updated, though; this matters for types that can be == without being identical. See BTreeMap::insert for more.

use nonempty_collections::nebtm;

let mut m = nebtm!["Vilya" => "Elrond", "Nenya" => "Galadriel"];
assert_eq!(None, m.insert("Narya", "Cirdan"));

// The Ring of Fire was given to Gandalf upon his arrival in Middle Earth.
assert_eq!(Some("Cirdan"), m.insert("Narya", "Gandalf"));