NEIndexSet

Struct NEIndexSet 

pub struct NEIndexSet<T, S = RandomState> { /* private fields */ }
A non-empty, growable IndexSet.

Construction and Access

The neis macro is the simplest way to construct an NEIndexSet:

use nonempty_collections::*;

let s = neis![1, 1, 2, 2, 3, 3, 4, 4];
let mut v: NEVec<_> = s.nonempty_iter().collect();
v.sort();
assert_eq!(nev![&1, &2, &3, &4], v);
use nonempty_collections::neis;

let s = neis!["Fëanor", "Fingolfin", "Finarfin"];
assert!(s.contains(&"Fëanor"));

Conversion

If you have a IndexSet but want an NEIndexSet, try NEIndexSet::try_from_set. Naturally, this might not succeed.

If you have an NEIndexSet but want a IndexSet, try their corresponding From instance. This will always succeed.

use indexmap::set::IndexSet;

use nonempty_collections::neis;

let n0 = neis![1, 2, 3];
let s0 = IndexSet::from(n0);

// Or just use `Into`.
let n1 = neis![1, 2, 3];
let s1: IndexSet<_> = n1.into();

API Differences with IndexSet

Note that the following methods aren’t implemented for NEIndexSet:

  • clear
  • drain
  • drain_filter
  • remove
  • retain
  • take

As these methods are all “mutate-in-place” style and are difficult to reconcile with the non-emptiness guarantee.

Implementations

Source

impl<T, S> NEIndexSet<T, S>

pub fn capacity(&self) -> NonZeroUsize

Returns the number of elements the set can hold without reallocating.

pub fn hasher(&self) -> &S

Returns a reference to the set’s BuildHasher.

pub fn iter(&self) -> Iter<'_, T>

Returns a regular iterator over the values in this non-empty set.

For a NonEmptyIterator see Self::nonempty_iter().

pub fn nonempty_iter(&self) -> Iter<'_, T>

An iterator visiting all elements in arbitrary order.

pub fn len(&self) -> NonZeroUsize

Returns the number of elements in the set. Always 1 or more.

use nonempty_collections::neis;

let s = neis![1, 2, 3];
assert_eq!(3, s.len().get());
pub const fn is_empty(&self) -> bool

👎Deprecated since 0.1.0:

A NEIndexSet is never empty.

A NEIndexSet is never empty.

Source

impl<T> NEIndexSet<T>
where T: Eq + Hash,

pub fn new(value: T) -> Self

Creates a new NEIndexSet with a single element.

pub fn with_capacity(capacity: NonZeroUsize, value: T) -> NEIndexSet<T>

Creates a new NEIndexSet with a single element and specified capacity.

use std::hash::RandomState;
use std::num::NonZeroUsize;

use nonempty_collections::*;
let set = NEIndexSet::with_capacity(NonZeroUsize::MIN, "hello");
assert_eq!(neis! {"hello"}, set);
assert!(set.capacity().get() >= 1);
Source

impl<T, S> NEIndexSet<T, S>
where T: Eq + Hash, S: BuildHasher,

pub fn try_from_set(set: IndexSet<T, S>) -> Option<NEIndexSet<T, S>>

Attempt a conversion from a IndexSet, consuming the given IndexSet. Will return None if the IndexSet is empty.

use indexmap::set::IndexSet;

use nonempty_collections::neis;
use nonempty_collections::NEIndexSet;

let mut s = IndexSet::new();
s.extend([1, 2, 3]);

let n = NEIndexSet::try_from_set(s);
assert_eq!(Some(neis![1, 2, 3]), n);
let s: IndexSet<()> = IndexSet::new();
assert_eq!(None, NEIndexSet::try_from_set(s));
pub fn contains<Q>(&self, value: &Q) -> bool
where T: Borrow<Q>, Q: Eq + Hash + ?Sized,

Returns true if the set contains a value.

use nonempty_collections::neis;

let s = neis![1, 2, 3];
assert!(s.contains(&3));
assert!(!s.contains(&10));
pub fn difference<'a>( &'a self, other: &'a NEIndexSet<T, S>, ) -> Difference<'a, T, S>

Visits the values representing the difference, i.e., the values that are in self but not in other.

use nonempty_collections::neis;

let s0 = neis![1, 2, 3];
let s1 = neis![3, 4, 5];
let mut v: Vec<_> = s0.difference(&s1).collect();
v.sort();
assert_eq!(vec![&1, &2], v);
pub fn get<Q>(&self, value: &Q) -> Option<&T>
where T: Borrow<Q>, Q: Eq + Hash,

Returns a reference to the value in the set, if any, that is equal to the given value.

The value may be any borrowed form of the set’s value type, but Hash and Eq on the borrowed form must match those for the value type.

use nonempty_collections::neis;

let s = neis![1, 2, 3];
assert_eq!(Some(&3), s.get(&3));
assert_eq!(None, s.get(&10));
pub fn insert(&mut self, value: T) -> bool

Adds a value to the set.

If the set did not have this value present, true is returned.

If the set did have this value present, false is returned.

use nonempty_collections::neis;

let mut s = neis![1, 2, 3];
assert_eq!(false, s.insert(2));
assert_eq!(true, s.insert(4));
pub fn intersection<'a>( &'a self, other: &'a NEIndexSet<T, S>, ) -> Intersection<'a, T, S>

Visits the values representing the interesection, i.e., the values that are both in self and other.

use nonempty_collections::neis;

let s0 = neis![1, 2, 3];
let s1 = neis![3, 4, 5];
let mut v: Vec<_> = s0.intersection(&s1).collect();
v.sort();
assert_eq!(vec![&3], v);
pub fn is_disjoint(&self, other: &NEIndexSet<T, S>) -> bool

Returns true if self has no elements in common with other. This is equivalent to checking for an empty intersection.

use nonempty_collections::neis;

let s0 = neis![1, 2, 3];
let s1 = neis![4, 5, 6];
assert!(s0.is_disjoint(&s1));
pub fn is_subset(&self, other: &NEIndexSet<T, S>) -> bool

Returns true if the set is a subset of another, i.e., other contains at least all the values in self.

use nonempty_collections::neis;

let sub = neis![1, 2, 3];
let sup = neis![1, 2, 3, 4];

assert!(sub.is_subset(&sup));
assert!(!sup.is_subset(&sub));
pub fn is_superset(&self, other: &NEIndexSet<T, S>) -> bool

Returns true if the set is a superset of another, i.e., self contains at least all the values in other.

use nonempty_collections::neis;

let sub = neis![1, 2, 3];
let sup = neis![1, 2, 3, 4];

assert!(sup.is_superset(&sub));
assert!(!sub.is_superset(&sup));
pub fn replace(&mut self, value: T) -> Option<T>

Adds a value to the set, replacing the existing value, if any, that is equal to the given one. Returns the replaced value.

pub fn reserve(&mut self, additional: usize)

Reserves capacity for at least additional more elements to be inserted in the NEIndexSet. The collection may reserve more space to avoid frequent reallocations.

Panics

Panics if the new allocation size overflows usize.

pub fn shrink_to_fit(&mut self)

Shrinks the capacity of the set as much as possible. It will drop down as much as possible while maintaining the internal rules and possibly leaving some space in accordance with the resize policy.

pub fn union<'a>(&'a self, other: &'a NEIndexSet<T, S>) -> Union<'a, T, S>

Visits the values representing the union, i.e., all the values in self or other, without duplicates.

Note that a Union is always non-empty.

use nonempty_collections::*;

let s0 = neis![1, 2, 3];
let s1 = neis![3, 4, 5];
let mut v: NEVec<_> = s0.union(&s1).collect();
v.sort();
assert_eq!(nev![&1, &2, &3, &4, &5], v);
pub fn with_capacity_and_hasher( capacity: NonZeroUsize, hasher: S, value: T, ) -> NEIndexSet<T, S>

pub fn with_hasher(hasher: S, value: T) -> NEIndexSet<T, S>