pub struct NEBTreeMap<K, V> { /* private fields */ }Expand description
A non-empty, growable BTreeMap.
use nonempty_collections::nebtm;
let m = nebtm!["elves" => 3000, "orcs" => 10000];
assert_eq!(2, m.len().get());Implementations
Sourceimpl<K, V> NEBTreeMap<K, V>where
K: Ord,
impl<K, V> NEBTreeMap<K, V>where
K: Ord,
Sourcepub fn new(k: K, v: V) -> NEBTreeMap<K, V>
pub fn new(k: K, v: V) -> NEBTreeMap<K, V>
Creates a new NEBTreeMap with a single element.
Sourceimpl<K, V> NEBTreeMap<K, V>
impl<K, V> NEBTreeMap<K, V>
Sourcepub fn try_from_map(map: BTreeMap<K, V>) -> Option<Self>
pub fn try_from_map(map: BTreeMap<K, V>) -> Option<Self>
Attempt a conversion from BTreeMap, consuming the given BTreeMap.
Will return None if the BTreeMap is empty.
use std::collections::*;
use nonempty_collections::*;
let mut map = BTreeMap::new();
map.extend([("a", 1), ("b", 2)]);
assert_eq!(Some(nebtm! {"a" => 1, "b" => 2}), NEBTreeMap::try_from_map(map));
let map: BTreeMap<(), ()> = BTreeMap::new();
assert_eq!(None, NEBTreeMap::try_from_map(map));Sourcepub fn iter(&self) -> Iter<'_, K, V>
pub fn iter(&self) -> Iter<'_, K, V>
Returns a regular iterator over the entries in this non-empty map.
For a NonEmptyIterator see Self::nonempty_iter().
Sourcepub fn iter_mut(&mut self) -> IterMut<'_, K, V>
pub fn iter_mut(&mut self) -> IterMut<'_, K, V>
Returns a regular mutable iterator over the entries in this non-empty map.
For a NonEmptyIterator see Self::nonempty_iter_mut().
Sourcepub fn nonempty_iter(&self) -> Iter<'_, K, V>
pub fn nonempty_iter(&self) -> Iter<'_, K, V>
An iterator visiting all elements in arbitrary order. The iterator
element type is (&'a K, &'a V).
Sourcepub fn nonempty_iter_mut(&mut self) -> IterMut<'_, K, V>
pub fn nonempty_iter_mut(&mut self) -> IterMut<'_, K, V>
An iterator visiting all elements in arbitrary order. The iterator
element type is (&'a K, &'a mut V).
Panics
If you manually advance this iterator until empty and then call first,
you’re in for a surprise.
Sourcepub fn keys(&self) -> Keys<'_, K, V>
pub fn keys(&self) -> Keys<'_, K, V>
An iterator visiting all keys in arbitrary order. The iterator element
type is &'a K.
use nonempty_collections::*;
let m = nebtm!["Valmar" => "Vanyar", "Tirion" => "Noldor", "Alqualondë" => "Teleri"];
let mut v: NEVec<_> = m.keys().collect();
v.sort();
assert_eq!(nev![&"Alqualondë", &"Tirion", &"Valmar"], v);Sourcepub fn len(&self) -> NonZeroUsize
pub fn len(&self) -> NonZeroUsize
Returns the number of elements in the map. Always 1 or more.
use nonempty_collections::nebtm;
let m = nebtm!["a" => 1, "b" => 2];
assert_eq!(2, m.len().get());Sourcepub fn values(&self) -> Values<'_, K, V>
pub fn values(&self) -> Values<'_, K, V>
An iterator visiting all values in arbitrary order. The iterator element
type is &'a V.
use nonempty_collections::*;
let m = nebtm!["Valmar" => "Vanyar", "Tirion" => "Noldor", "Alqualondë" => "Teleri"];
let mut v: NEVec<_> = m.values().collect();
v.sort();
assert_eq!(nev![&"Noldor", &"Teleri", &"Vanyar"], v);Sourceimpl<K, V> NEBTreeMap<K, V>where
K: Ord,
impl<K, V> NEBTreeMap<K, V>where
K: Ord,
Sourcepub fn contains_key<Q>(&self, k: &Q) -> bool
pub fn contains_key<Q>(&self, k: &Q) -> bool
Returns true if the map contains a value.
use nonempty_collections::nebtm;
let m = nebtm!["Jack" => 8];
assert!(m.contains_key("Jack"));
assert!(!m.contains_key("Colin"));Sourcepub fn get<Q>(&self, k: &Q) -> Option<&V>
pub fn get<Q>(&self, k: &Q) -> Option<&V>
Returns a reference to the value corresponding to the key.
The key may be any borrowed form of the map’s value type, but Hash and
Eq on the borrowed form must match those for the key type.
use nonempty_collections::nebtm;
let m = nebtm!["silmarils" => 3];
assert_eq!(Some(&3), m.get("silmarils"));
assert_eq!(None, m.get("arkenstone"));Sourcepub fn get_key_value<Q>(&self, k: &Q) -> Option<(&K, &V)>
pub fn get_key_value<Q>(&self, k: &Q) -> Option<(&K, &V)>
Returns the key-value pair corresponding to the key.
The key may be any borrowed form of the map’s value type, but Hash and
Eq on the borrowed form must match those for the key type.
use nonempty_collections::nebtm;
let m = nebtm!["silmarils" => 3];
assert_eq!(Some((&"silmarils", &3)), m.get_key_value("silmarils"));
assert_eq!(None, m.get_key_value("arkenstone"));Sourcepub fn get_mut<Q>(&mut self, k: &Q) -> Option<&mut V>
pub fn get_mut<Q>(&mut self, k: &Q) -> Option<&mut V>
Returns a reference to the value corresponding to the key.
The key may be any borrowed form of the map’s value type, but Hash and
Eq on the borrowed form must match those for the key type.
use nonempty_collections::nebtm;
let mut m = nebtm!["silmarils" => 3];
let mut v = m.get_mut("silmarils").unwrap();
// And thus it came to pass that the Silmarils found their long homes:
// one in the airs of heaven, and one in the fires of the heart of the
// world, and one in the deep waters.
*v -= 3;
assert_eq!(Some(&0), m.get("silmarils"));Sourcepub fn insert(&mut self, k: K, v: V) -> Option<V>
pub fn insert(&mut self, k: K, v: V) -> Option<V>
Insert a key-value pair into the map.
If the map did not have this present, None is returned.
If the map did have this key present, the value is updated, and the old
value is returned. The key is not updated, though; this matters for
types that can be == without being identical. See BTreeMap::insert
for more.
use nonempty_collections::nebtm;
let mut m = nebtm!["Vilya" => "Elrond", "Nenya" => "Galadriel"];
assert_eq!(None, m.insert("Narya", "Cirdan"));
// The Ring of Fire was given to Gandalf upon his arrival in Middle Earth.
assert_eq!(Some("Cirdan"), m.insert("Narya", "Gandalf"));Trait Implementations
Sourceimpl<K, V> AsMut<BTreeMap<K, V>> for NEBTreeMap<K, V>
impl<K, V> AsMut<BTreeMap<K, V>> for NEBTreeMap<K, V>
Sourceimpl<K, V> AsRef<BTreeMap<K, V>> for NEBTreeMap<K, V>
impl<K, V> AsRef<BTreeMap<K, V>> for NEBTreeMap<K, V>
Sourceimpl<K: Clone, V: Clone> Clone for NEBTreeMap<K, V>
impl<K: Clone, V: Clone> Clone for NEBTreeMap<K, V>
Sourcefn clone(&self) -> NEBTreeMap<K, V>
fn clone(&self) -> NEBTreeMap<K, V>
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl<'de, K, V> Deserialize<'de> for NEBTreeMap<K, V>
impl<'de, K, V> Deserialize<'de> for NEBTreeMap<K, V>
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
impl<K, V> Eq for NEBTreeMap<K, V>
Sourceimpl<K, V> Extend<(K, V)> for NEBTreeMap<K, V>where
K: Ord,
impl<K, V> Extend<(K, V)> for NEBTreeMap<K, V>where
K: Ord,
Sourcefn extend<I: IntoIterator<Item = (K, V)>>(&mut self, iter: I)
fn extend<I: IntoIterator<Item = (K, V)>>(&mut self, iter: I)
Sourcefn extend_one(&mut self, item: A)
fn extend_one(&mut self, item: A)
extend_one)Sourcefn extend_reserve(&mut self, additional: usize)
fn extend_reserve(&mut self, additional: usize)
extend_one)Sourceimpl<K, V> From<NEBTreeMap<K, V>> for BTreeMap<K, V>where
K: Ord,
impl<K, V> From<NEBTreeMap<K, V>> for BTreeMap<K, V>where
K: Ord,
Sourcefn from(m: NEBTreeMap<K, V>) -> Self
fn from(m: NEBTreeMap<K, V>) -> Self
use nonempty_collections::nebtm;
use std::collections::BTreeMap;
let m: BTreeMap<&str, usize> = nebtm!["population" => 1000].into();
assert!(m.contains_key("population"));Sourceimpl<K, V> FromNonEmptyIterator<(K, V)> for NEBTreeMap<K, V>where
K: Ord,
use nonempty_collections::*;
let v = nev![('a', 1), ('b', 2), ('c', 3), ('a', 4)];
let m0: NEBTreeMap<_, _> = v.into_nonempty_iter().collect();
let m1: NEBTreeMap<_, _> = nebtm!['a' => 4, 'b' => 2, 'c' => 3];
assert_eq!(m0, m1);
impl<K, V> FromNonEmptyIterator<(K, V)> for NEBTreeMap<K, V>where
K: Ord,
use nonempty_collections::*;
let v = nev![('a', 1), ('b', 2), ('c', 3), ('a', 4)];
let m0: NEBTreeMap<_, _> = v.into_nonempty_iter().collect();
let m1: NEBTreeMap<_, _> = nebtm!['a' => 4, 'b' => 2, 'c' => 3];
assert_eq!(m0, m1);Sourcefn from_nonempty_iter<I>(iter: I) -> Selfwhere
I: IntoNonEmptyIterator<Item = (K, V)>,
fn from_nonempty_iter<I>(iter: I) -> Selfwhere
I: IntoNonEmptyIterator<Item = (K, V)>,
NonEmptyIterator.Sourceimpl<K, V> IntoIterator for NEBTreeMap<K, V>
impl<K, V> IntoIterator for NEBTreeMap<K, V>
Sourceimpl<'a, K, V> IntoIterator for &'a NEBTreeMap<K, V>
impl<'a, K, V> IntoIterator for &'a NEBTreeMap<K, V>
Sourceimpl<'a, K, V> IntoIterator for &'a mut NEBTreeMap<K, V>
impl<'a, K, V> IntoIterator for &'a mut NEBTreeMap<K, V>
Sourceimpl<K, V> IntoNonEmptyIterator for NEBTreeMap<K, V>
impl<K, V> IntoNonEmptyIterator for NEBTreeMap<K, V>
Sourcetype IntoNEIter = IntoIter<K, V>
type IntoNEIter = IntoIter<K, V>
NonEmptyIterator are we turning this into?Sourcefn into_nonempty_iter(self) -> Self::IntoNEIter
fn into_nonempty_iter(self) -> Self::IntoNEIter
NonEmptyIterator from a value.Sourceimpl<'a, K, V> IntoNonEmptyIterator for &'a NEBTreeMap<K, V>
impl<'a, K, V> IntoNonEmptyIterator for &'a NEBTreeMap<K, V>
Sourcetype IntoNEIter = Iter<'a, K, V>
type IntoNEIter = Iter<'a, K, V>
NonEmptyIterator are we turning this into?Sourcefn into_nonempty_iter(self) -> Self::IntoNEIter
fn into_nonempty_iter(self) -> Self::IntoNEIter
NonEmptyIterator from a value.Sourceimpl<K, V> PartialEq for NEBTreeMap<K, V>
impl<K, V> PartialEq for NEBTreeMap<K, V>
Sourceimpl<K, V> Serialize for NEBTreeMap<K, V>
impl<K, V> Serialize for NEBTreeMap<K, V>
Sourceimpl<K, V> Singleton for NEBTreeMap<K, V>where
K: Ord,
impl<K, V> Singleton for NEBTreeMap<K, V>where
K: Ord,